package de.hamster.debugger.model;

import de.hamster.model.CreateInstruction;
import de.hamster.model.HamsterInitialisierungsException;
import de.hamster.model.HamsterInitializationException;
import de.hamster.model.HamsterInstruction;
import de.hamster.model.HamsterNichtInitialisiertException;
import de.hamster.model.HamsterNotInitializedException;
import de.hamster.model.InstructionProcessor;
import de.hamster.model.KachelLeerException;
import de.hamster.model.MauerDaException;
import de.hamster.model.MaulLeerException;
import de.hamster.model.MouthEmptyException;
import de.hamster.model.WallInFrontException;
import de.hamster.model.TileEmptyException;

import de.hamster.workbench.Utils;

/**
 * Dies ist die Oberklasse eines objektorientierten Hamsters.
 * 
 * @author $Author: djasper $
 * @version $Revision: 1.2 $
 */
public class Hamster extends IHamster {
	private int reihe;

	private int spalte;

	private int anzahlKoerner;

	private int blickrichtung;

	private boolean initialisiert = false;

	static int count = 0;

	static Hamster standard;

	public final static int NORD = 0;

	public final static int OST = 1;

	public final static int SUED = 2;

	public final static int WEST = 3;

	public final static int NORTH = 0;

	public final static int EAST = 1;

	public final static int SOUTH = 2;

	static void setProcessor(InstructionProcessor p) {
		processor = p;
	}

	public static Hamster getStandardHamster() {
		if (standard == null) {
			standard = new Hamster();
			standard.id = -1;
			Territorium.addHamster(standard);
			standard.initialisiert = true;
			int[] data = (int[]) processor.process(new HamsterInstruction(
					HamsterInstruction.GET_DATA, -1));
			standard.reihe = data[0];
			standard.spalte = data[1];
			standard.blickrichtung = data[2];
			standard.anzahlKoerner = data[3];
		}
		return standard;
	}

	public static Hamster getDefaultHamster() {
		return getStandardHamster();
	}

	public static int getAnzahlHamster() {
		return Territorium.getAnzahlHamster();
	}

	public static int getNumberOfHamsters() {
		return getAnzahlHamster();
	}

	public Hamster() {
	}

	public Hamster(int reihe, int spalte, int blickrichtung, int anzahlKoerner) {
		init(reihe, spalte, blickrichtung, anzahlKoerner);
	}

	public Hamster(Hamster h) {
		init(h.reihe, h.spalte, h.blickrichtung, h.anzahlKoerner);
	}

	public void init(int reihe, int spalte, int blickrichtung, int anzahlKoerner) {
		if (initialisiert)
			return;
		id = count++;
		Territorium.addHamster(this);
		Object o = processor.process(new CreateInstruction(id, reihe, spalte,
				blickrichtung, anzahlKoerner));
		if (o instanceof HamsterInitialisierungsException) {
			throw new HamsterInitializationException(
					new HamsterInitialisierungsException(this));
		} else {
			initialisiert = true;
			this.reihe = reihe;
			this.spalte = spalte;
			this.blickrichtung = blickrichtung;
			this.anzahlKoerner = anzahlKoerner;
		}
	}

	public void vor() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		try {
			super.vor();
			switch (blickrichtung) {
			case NORD:
				reihe--;
				break;
			case OST:
				spalte++;
				break;
			case SUED:
				reihe++;
				break;
			case WEST:
				spalte--;
				break;
			}
		} catch (MauerDaException e) {
			e.setHamster(this);
			throw new WallInFrontException(e);
		}
	}

	public void linksUm() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		super.linksUm();
		blickrichtung = (blickrichtung + 3) % 4;
	}

	public void nimm() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		try {
			super.nimm();
			anzahlKoerner++;
		} catch (KachelLeerException e) {
			e.setHamster(this);
			throw new TileEmptyException(e);
		}
	}

	public void gib() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		try {
			super.gib();
			anzahlKoerner--;
		} catch (MaulLeerException e) {
			e.setHamster(this);
			throw new MouthEmptyException(e);
		}
	}

	public boolean vornFrei() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		return super.vornFrei();
	}

	public boolean kornDa() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		return super.kornDa();
	}

	public boolean maulLeer() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		return super.maulLeer();
	}

	public int getReihe() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		return ((Integer) processor.process(new HamsterInstruction(
				HamsterInstruction.GET_ROW, id))).intValue();
	}

	public int getRow() {
		return getReihe();
	}

	public int getSpalte() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		return ((Integer) processor.process(new HamsterInstruction(
				HamsterInstruction.GET_COL, id))).intValue();
	}

	public int getColumn() {
		return getSpalte();
	}

	public int getBlickrichtung() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		return ((Integer) processor.process(new HamsterInstruction(
				HamsterInstruction.GET_DIR, id))).intValue();
	}

	public int getDirection() {
		return getBlickrichtung();
	}

	public int getAnzahlKoerner() {
		if (!initialisiert)
			throw new HamsterNotInitializedException(
					new HamsterNichtInitialisiertException(this));
		return ((Integer) processor.process(new HamsterInstruction(
				HamsterInstruction.GET_MOUTH, id))).intValue();
	}

	public int getNumberOfGrains() {
		return getAnzahlKoerner();
	}

	// Methode, die ueberprueft, ob die Werte der Attribute des aufgerufenen
	// Hamsters gleich der Attributwerte des uebergebenen Hamsters sind (zwei
	// nicht initialisierte Hamster sind auch gleich) hamster: muss ein Objekt
	// der Klasse Hamster oder einer davon abgeleiteten Klasse sein
	// (ueberschreibt die entsprechende von der Klasse Object geerbte Methode)
	public boolean equals(Object o) {
		if (!(o instanceof Hamster))
			return false;
		Hamster h = (Hamster) o;
		if (!h.initialisiert && !initialisiert)
			return true;
		else if (!initialisiert)
			return false;
		else if (!h.initialisiert)
			return false;
		return h.reihe == reihe && h.spalte == spalte
				&& h.anzahlKoerner == anzahlKoerner
				&& h.blickrichtung == blickrichtung;
	}

	// (ueberschreibt die entsprechende von der Klasse Object geerbte Methode)
	public int hashCode() {
		int hash = 7;
		hash = 31 * hash + this.reihe;
		hash = 31 * hash + this.spalte;
		hash = 31 * hash + this.blickrichtung;
		hash = 31 * hash + this.anzahlKoerner;
		return hash;
	}

	// Methode, die eine String-Repraesentation der folgenden Art fuer den
	// aufgerufenen Hamster liefert:
	// "Hamster steht auf Kachel (0/0) mit Blickrichtung OST und 2 Koernern im
	// Maul"
	// Wenn der aufgerufene Hamster noch nicht initialisiert ist, wird
	// folgender String geliefert: "Hamster ist nicht initialisiert"
	// (ueberschreibt die entsprechende von der Klasse Object geerbte Methode)
	public String toString() {
		if (Utils.language.equals("en")) {
			String[] namen = new String[] { "NORTH", "EAST", "SOUTH", "WEST" };
			String s = "Hamster on tile (" + reihe + "/" + spalte + ") ";
			s += "looking in direction " + namen[blickrichtung] + " with ";
			s += anzahlKoerner + (anzahlKoerner == 1 ? " corn" : " corn");
			return s + " in its mouth";
		} else {
			String[] namen = new String[] { "NORD", "OST", "SUED", "WEST" };
			String s = "Hamster steht auf Kachel (" + reihe + "/" + spalte
					+ ") ";
			s += "mit Blickrichtung " + namen[blickrichtung] + " und ";
			s += anzahlKoerner + (anzahlKoerner == 1 ? " Korn" : " Koernern");
			return s + " im Maul";
		}
	}
}
