package de.hamster.debugger.view;

import java.awt.Dimension;
import java.util.Observable;
import java.util.Observer;

import javax.swing.Box;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSlider;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.hamster.debugger.controller.DebuggerController;
import de.hamster.debugger.model.DebuggerModel;
import de.hamster.model.HamsterFile;
import de.hamster.workbench.ForwardAction;
import de.hamster.workbench.Utils;

/**
 * @author $Author: djasper $
 * @version $Revision: 1.1 $
 */
public class DebuggerTools implements Observer, ChangeListener {
	private DebuggerModel model;

	private StartAction startAction = new StartAction();

	public class StartAction extends ForwardAction {
		public StartAction() {
			super("debugger.start", DebuggerController.ACTION_START);
		}
	}

	private StepAction stepAction = new StepAction();

	public class StepAction extends ForwardAction {
		public StepAction() {
			super("debugger.stepinto", DebuggerController.ACTION_STEP);
		}
	}

	private StepOverAction stepOverAction = new StepOverAction();

	public class StepOverAction extends ForwardAction {
		public StepOverAction() {
			super("debugger.stepover", DebuggerController.ACTION_STEPOVER);
		}
	}

	private EnableAction enableAction = new EnableAction();

	public class EnableAction extends ForwardAction {
		public EnableAction() {
			super("debugger.enable", DebuggerController.ACTION_ENABLE);
		}
	}

	private StopAction stopAction = new StopAction();

	public class StopAction extends ForwardAction {
		public StopAction() {
			super("debugger.stop", DebuggerController.ACTION_STOP);
		}
	}

	private PauseAction pauseAction = new PauseAction();

	public class PauseAction extends ForwardAction {
		public PauseAction() {
			super("debugger.pause", DebuggerController.ACTION_PAUSE);
		}
	}

	private JToggleButton enableButton;

	private JCheckBoxMenuItem enableItem;

	private JSlider delay, delaySim;

	protected HamsterFile activeFile;

	public DebuggerTools(DebuggerModel model, DebuggerController controller) {
		this.model = model;
		model.addObserver(this);

		JMenu menu = controller.getWorkbench().getView().findMenu("editor",
				"debugger");
		menu.add(new JMenuItem(startAction));
		menu.add(new JMenuItem(pauseAction));
		menu.add(new JMenuItem(stopAction));
		menu.addSeparator();
		enableItem = new JCheckBoxMenuItem(enableAction);
		menu.add(enableItem);
		menu.add(new JMenuItem(stepAction));
		menu.add(new JMenuItem(stepOverAction));

		JToolBar toolBar = controller.getWorkbench().getView().findToolBar(
				"editor");
		toolBar.add(Box.createRigidArea(new Dimension(11, 11)));
		toolBar.add(Utils.createButton(startAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(pauseAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(stopAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		enableButton = Utils.createToggleButton(enableAction);
		toolBar.add(enableButton);
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(stepAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(stepOverAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		delay = new JSlider(0, 10, 5);
		delay.setToolTipText(Utils.getResource("debugger.delay.tooltip"));
		delay.addChangeListener(controller);
		delay.setInverted(true);
		delay.setSnapToTicks(true);
		delay.setMajorTickSpacing(1);
		delay.setPaintTicks(true);
		delay.addChangeListener(this);
		delay.setName("delay");
		toolBar.add(delay);

		JToolBar simulationBar = controller.getWorkbench().getView()
				.findToolBar("simulation");
		simulationBar.add(Box.createRigidArea(new Dimension(11, 11)));
		simulationBar.add(Utils.createButton(startAction));
		simulationBar.add(Box.createRigidArea(new Dimension(2, 2)));
		simulationBar.add(Utils.createButton(pauseAction));
		simulationBar.add(Box.createRigidArea(new Dimension(2, 2)));
		simulationBar.add(Utils.createButton(stopAction));
		delaySim = new JSlider(0, 10, 5);
		delaySim.setToolTipText(Utils.getResource("debugger.delay.tooltip"));
		delaySim.addChangeListener(controller);
		delaySim.setInverted(true);
		delaySim.setSnapToTicks(true);
		delaySim.setMajorTickSpacing(1);
		delaySim.setPaintTicks(true);
		delaySim.setName("sim");
		delaySim.addChangeListener(this);
		simulationBar.add(Box.createRigidArea(new Dimension(2, 2)));
		simulationBar.add(delaySim);

		startAction.addActionListener(controller);
		stepAction.addActionListener(controller);
		stepOverAction.addActionListener(controller);
		stopAction.addActionListener(controller);
		enableAction.addActionListener(controller);
		pauseAction.addActionListener(controller);

		updateButtonStates();
	}

	public void update(Observable o, Object arg) {
		updateButtonStates();
		if (arg == DebuggerModel.ARG_ENABLE) {
			enableButton.setSelected(model.isEnabled());
			enableItem.setSelected(model.isEnabled());
		}
	}

	public void updateButtonStates() {

		// martin
		// Falls die aktive Datei ein Scheme-Programm ist, muessen
		// ein paar Buttons ausgeblendet werden.
		if (activeFile != null
				&& activeFile.getType() == HamsterFile.SCHEMEPROGRAM) {
			enableAction.setEnabled(false);
			stopAction
					.setEnabled(model.getState() != DebuggerModel.NOT_RUNNING);
			pauseAction.setEnabled(false);
			stepAction.setEnabled(false);
			stepOverAction.setEnabled(false);
			startAction
					.setEnabled(activeFile != null
							&& (model.getState() != DebuggerModel.RUNNING && !(activeFile
									.getType() == HamsterFile.HAMSTERCLASS && model
									.getState() == DebuggerModel.NOT_RUNNING)));
			if (de.hamster.workbench.Workbench.workbench != null) {
				de.hamster.workbench.Workbench.getWorkbench().getSimulation()
						.getSimulationTools().getResetAction()
						.setEnabled(false);
			}
		} else {
			startAction
					.setEnabled(activeFile != null
							&& (model.getState() != DebuggerModel.RUNNING && !(activeFile
									.getType() == HamsterFile.HAMSTERCLASS && model
									.getState() == DebuggerModel.NOT_RUNNING)));
			pauseAction.setEnabled(model.getState() == DebuggerModel.RUNNING);
			stepAction.setEnabled(model.isEnabled()
					&& (model.getState() == DebuggerModel.PAUSED || (model
							.getState() == DebuggerModel.NOT_RUNNING)
							&& startAction.isEnabled()));
			stepOverAction.setEnabled(model.isEnabled()
					&& model.getState() == DebuggerModel.PAUSED);
			stopAction
					.setEnabled(model.getState() != DebuggerModel.NOT_RUNNING);
			enableAction.setEnabled(!Utils.runlocally);
			if (de.hamster.workbench.Workbench.workbench != null) {
				de.hamster.workbench.Workbench
						.getWorkbench()
						.getSimulation()
						.getSimulationTools()
						.getResetAction()
						.setEnabled(
								de.hamster.workbench.Workbench.getWorkbench()
										.getSimulation().getSimulationModel().savedTerrain != null
										&& activeFile != null
										&& (model.getState() != DebuggerModel.RUNNING
												&& model.getState() != DebuggerModel.PAUSED && !(activeFile
												.getType() == HamsterFile.HAMSTERCLASS && model
												.getState() == DebuggerModel.NOT_RUNNING)));
			}
		}

	}

	public void setActiveFile(HamsterFile activeFile) {
		this.activeFile = activeFile;
		updateButtonStates();
	}

	public void stateChanged(ChangeEvent e) {
		JSlider slider = (JSlider) e.getSource();
		if (slider == delay) {
			if (delaySim.getValue() != delay.getValue()) {
				delaySim.setValue(delay.getValue());
			}
		} else if (slider == delaySim) {
			if (delay.getValue() != delaySim.getValue()) {
				delay.setValue(delaySim.getValue());
			}
		}
	}
}
