package de.hamster.editor.view;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

import de.hamster.model.HamsterFile;
import de.hamster.workbench.Utils;

/**
 * Diese Klasse stellt einen Dialog dar, in dem der Benutzer den Typ eines neuen
 * Hamsterprogramms auswaehlen kann. Dies ist entweder ein imperatives Programm,
 * ein objektorientiertes Hamsterprogramm oder eine einfach Java-Klasse.
 * 
 * @author Daniel Jasper
 */
public class NewHamsterDialog extends JDialog {
	public class ProgramType {
		String string;

		char type;

		public ProgramType(String string, char type) {
			this.string = string;
			this.type = type;
		}

		public String toString() {
			return string;
		}
	}

	protected ProgramType[] programTypes = null;

	protected JOptionPane optionPane;

	protected JComboBox types;

	protected Component parent;

	public NewHamsterDialog(Component parent) {
		super(JOptionPane.getFrameForComponent(parent), "Neu", true);
		this.parent = parent;

		if (Utils.SCHEME) {
			programTypes = new ProgramType[4];
			programTypes[0] = new ProgramType(Utils.getResource("editor.impp"), HamsterFile.IMPERATIVE);
			programTypes[1] = new ProgramType(Utils.getResource("editor.oop"), HamsterFile.OBJECT);
			programTypes[2] = new ProgramType(Utils.getResource("editor.klasse"), HamsterFile.HAMSTERCLASS);
			programTypes[3] = new ProgramType(Utils.getResource("editor.schemep"), HamsterFile.SCHEMEPROGRAM); // Martin
			//programTypes[4] = new ProgramType("Scheme Konsole", HamsterFile.SCHEMEKONSOLE); // Martin
		} else {
			programTypes = new ProgramType[3];
			programTypes[0] = new ProgramType(Utils.getResource("editor.impp"), HamsterFile.IMPERATIVE);
			programTypes[1] = new ProgramType(Utils.getResource("editor.oop"), HamsterFile.OBJECT);
			programTypes[2] = new ProgramType(Utils.getResource("editor.klasse"), HamsterFile.HAMSTERCLASS);
		}

		types = new JComboBox(programTypes);
		optionPane = new JOptionPane();
		optionPane.setMessageType(JOptionPane.QUESTION_MESSAGE);
		optionPane.setMessage(new Object[] { Utils.getResource("editor.programmtyp"), types });
		optionPane.setOptionType(JOptionPane.OK_CANCEL_OPTION);
		optionPane.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				String prop = e.getPropertyName();
				if (isVisible() && (e.getSource() == optionPane)
						&& (prop.equals(JOptionPane.VALUE_PROPERTY))) {
					setVisible(false);
				}
			}
		});

		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		setContentPane(optionPane);
		pack();
	}

	public char askForType() {
		optionPane.setValue(null);
		setLocationRelativeTo(parent);
		setVisible(true);
		int value = ((Integer) optionPane.getValue()).intValue();
		if (value == JOptionPane.YES_OPTION)
			return ((ProgramType) types.getSelectedItem()).type;
		else
			return (char) -1;
	}
}