package de.hamster.simulation.view;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import de.hamster.simulation.model.Terminal;
import de.hamster.workbench.Utils;

/**
 * @author $Author: djasper $
 * @version $Revision: 1.4 $
 */
public class DialogTerminal extends JDialog implements Terminal {
	private static DialogTerminal instance;

	protected JOptionPane optionPane;

	protected JTextField textField;

	protected JFrame parent;

	public static DialogTerminal createInstance(JFrame parent) {
		instance = new DialogTerminal(parent);
		return instance;
	}

	public static DialogTerminal getInstance() {
		return instance;
	}

	private DialogTerminal(JFrame parent) {
		super(parent);
		this.parent = parent;
		optionPane = new JOptionPane();
		optionPane.setOptionType(JOptionPane.DEFAULT_OPTION);
		optionPane.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				String prop = e.getPropertyName();
				if (isVisible() && (e.getSource() == optionPane)
						&& (prop.equals(JOptionPane.VALUE_PROPERTY))) {
					close();
				}
			}
		});

		textField = new JTextField();

		setContentPane(optionPane);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}

	private JLabel getMessage(int hamsterid, String string) {
		string = "<html>" + string + "</html>";
		string = string.replaceAll("\n", "<br>");
		JLabel label = new JLabel(string);
		label.setForeground(Utils.COLORS[Math.min(hamsterid + 1,
				Utils.COLORS.length - 1)]);
		return label;
	}

	public synchronized void showAndWaitForClose() {
		pack();
		setVisible(true);
		this.setLocationRelativeTo(parent);
		try {
			while (isVisible())
				wait();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		optionPane.setValue(null);
	}

	public void write(int hamsterid, String string) {
		setTitle(Utils.getResource("hamster.schreib") + "()");
		optionPane.setMessage(getMessage(hamsterid, string));
		optionPane.setMessageType(JOptionPane.INFORMATION_MESSAGE);
		showAndWaitForClose();
	}

	public void write(Throwable e) {
		setTitle("Exception");
		String s = e.toString();
		StringWriter sw = new StringWriter();
		StackTraceElement[] ste = e.getStackTrace();
		for (int i = 0; i < ste.length - 1; i++) {
			if (ste[i].getClassName().equals(
					"de.hamster.debugger.model.Hamster"))
				continue;
			if (ste[i].getClassName().equals(
					"de.hamster.debugger.model.IHamster"))
				continue;
			s += "\n   in   " + ste[i].getClassName() + "."
					+ ste[i].getMethodName() + " ("
					+ Utils.getResource("simulation.view.reihe") + " "
					+ ste[i].getLineNumber() + ")";
		}
		e.printStackTrace(new PrintWriter(sw));
		optionPane.setMessage(s);
		optionPane.setMessageType(JOptionPane.INFORMATION_MESSAGE);
		showAndWaitForClose();
	}

	public int readInt(int hamsterid, String message) {
		setTitle(Utils.getResource("hamster.liesZahl") + "()");
		textField.setText("0");
		optionPane.setMessage(new Object[] { getMessage(hamsterid, message),
				textField });
		optionPane.setMessageType(JOptionPane.QUESTION_MESSAGE);
		showAndWaitForClose();
		try {
			return Integer.parseInt(textField.getText());
		} catch (NumberFormatException e) {
		}
		return 0;
	}

	public String readString(int hamsterid, String message) {
		setTitle(Utils.getResource("hamster.liesZeichenkette") + "()");
		textField.setText("");
		optionPane.setMessage(new Object[] { getMessage(hamsterid, message),
				textField });
		optionPane.setMessageType(JOptionPane.QUESTION_MESSAGE);
		showAndWaitForClose();
		return textField.getText();
	}

	public synchronized void close() {
		setVisible(false);
		notify();
	}
}