package de.hamster.simulation.view;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import de.hamster.simulation.controller.SimulationController;
import de.hamster.simulation.model.SimulationModel;
import de.hamster.workbench.ForwardAction;
import de.hamster.workbench.Utils;

public class SimulationTools implements MouseListener, MouseMotionListener {
	public static final int WALL = 0;

	private int startRow;
	private int startCol;
	private int endRow;
	private int endCol;
	private Rectangle lastDrag;
	protected boolean isDragging;

	private TerrainTool activeTool;
	private SimulationPanel simulationPanel;

	private SimulationModel simulationModel;
	private SimulationController simulationController;

	public class NewAction extends ForwardAction {
		public NewAction() {
			super("simulation.new", SimulationController.NEW);
		}
	}
	public class OpenAction extends ForwardAction {
		public OpenAction() {
			super("simulation.open", SimulationController.OPEN);
		}
	}
	public class SaveAsAction extends ForwardAction {
		public SaveAsAction() {
			super("simulation.save", SimulationController.SAVE_AS);
		}
	}
	public class ZoomInAction extends AbstractAction {
		public ZoomInAction() {
			super();
			Utils.setData(this, "simulation.zoomin");
		}
		public void actionPerformed(ActionEvent e) {
			simulationPanel.zoomIn();
		}
	}
	public class ZoomOutAction extends AbstractAction {
		public ZoomOutAction() {
			super();
			Utils.setData(this, "simulation.zoomout");
		}
		public void actionPerformed(ActionEvent e) {
			simulationPanel.zoomOut();
		}
	}

	public class SelectCornAction extends AbstractAction {
		public SelectCornAction() {
			Utils.setData(this, "simulation.corn");
		}
		public void actionPerformed(ActionEvent e) {
			activeTool = cornTool;
		}
	}
	public class SelectDeleteAction extends AbstractAction {
		public SelectDeleteAction() {
			Utils.setData(this, "simulation.delete");
		}
		public void actionPerformed(ActionEvent e) {
			activeTool = deleteTool;
		}
	}
	public class SelectHamsterAction extends AbstractAction {
		public SelectHamsterAction() {
			Utils.setData(this, "simulation.hamster");
		}
		public void actionPerformed(ActionEvent e) {
			activeTool = hamsterTool;
		}
	}
	public class SelectWallAction extends AbstractAction {
		public SelectWallAction() {
			Utils.setData(this, "simulation.wall");
		}
		public void actionPerformed(ActionEvent e) {
			activeTool = wallTool;
		}
	}
	public class TurnHamsterAction extends ForwardAction {
		public TurnHamsterAction() {
			super("simulation.turn", SimulationController.TURN);
		}
	}
	public class HamsterCornAction extends ForwardAction {
		public HamsterCornAction() {
			super("simulation.hamstercorn", SimulationController.HAMSTER_CORN);
		}
	}

	public class ResetAction extends ForwardAction {
		public ResetAction() {
			super("simulation.reset", SimulationController.RESET);
			this.setEnabled(false);
		}
	}

	private NewAction newAction = new NewAction();
	private OpenAction openAction = new OpenAction();
	private SaveAsAction saveAsAction = new SaveAsAction();
	private SelectCornAction selectCornAction = new SelectCornAction();
	private SelectHamsterAction selectHamsterAction = new SelectHamsterAction();
	private SelectWallAction selectWallAction = new SelectWallAction();
	private SelectDeleteAction selectDeleteAction = new SelectDeleteAction();
	private ZoomInAction zoomInAction = new ZoomInAction();
	private ZoomOutAction zoomOutAction = new ZoomOutAction();
	private TurnHamsterAction turnHamsterAction = new TurnHamsterAction();
	private HamsterCornAction hamsterCornAction = new HamsterCornAction();
	private ResetAction resetAction = new ResetAction();
	
	// dibo
	public ResetAction getResetAction() {
		return resetAction;
	}

	public class CornTool extends TerrainTool {
		public CornTool() {
			super(true);
			setImage(Utils.getImage("Corn32.png"));
		}
		public void done() {
			try {
				if (!simulationController.containsValidCornCell(Math.min(
						startCol, endCol), Math.min(startRow, endRow), Math
						.max(startCol, endCol), Math.max(startRow, endRow)))
					return;
				String countString = Utils.input(simulationPanel,
						"simulation.dialog.cornnumber", "1");
				if (countString != null) {
					int count = Math.max(Integer.parseInt(countString), 0);
					simulationController.setCorn(Math.min(startCol, endCol),
							Math.min(startRow, endRow), Math.max(startCol,
									endCol), Math.max(startRow, endRow), count);
				}
			} catch (NumberFormatException e) {
			}
		}
	}
	public class HamsterTool extends TerrainTool {
		public HamsterTool() {
			super(false);
		}
		public void done() {
			simulationController.setHamsterPos(endCol, endRow);
		}
	}
	public class WallTool extends TerrainTool {
		public WallTool() {
			super(true);
			setImage(Utils.getImage("Wall32.png"));
		}
		public void done() {
			simulationController.setWall(Math.min(startCol, endCol), Math.min(
					startRow, endRow), Math.max(startCol, endCol), Math.max(
					startRow, endRow));
		}
	}
	public class DeleteTool extends TerrainTool {
		public DeleteTool() {
			super(true);
			setImage(Utils.getImage("Delete32.gif"));
		}
		public void done() {
			simulationController.setEmpty(Math.min(startCol, endCol), Math.min(
					startRow, endRow), Math.max(startCol, endCol), Math.max(
					startRow, endRow));
		}
	}
	private CornTool cornTool = new CornTool();
	private DeleteTool deleteTool = new DeleteTool();
	private HamsterTool hamsterTool = new HamsterTool();
	private WallTool wallTool = new WallTool();

	public SimulationTools(SimulationModel model,
			SimulationController controller) {
		this.simulationModel = model;
		this.simulationController = controller;
		newAction.addActionListener(controller);
		openAction.addActionListener(controller);
		saveAsAction.addActionListener(controller);
		turnHamsterAction.addActionListener(controller);
		hamsterCornAction.addActionListener(controller);
		resetAction.addActionListener(controller);

		JToolBar toolBar = controller.getWorkbench().getView().findToolBar(
				"simulation");
		toolBar.add(Utils.createButton(newAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(openAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(saveAsAction));

		toolBar.add(Box.createRigidArea(new Dimension(11, 11)));

		JToggleButton corn = Utils.createToggleButton(selectCornAction);
		JToggleButton hamster = Utils.createToggleButton(selectHamsterAction);
		JToggleButton wall = Utils.createToggleButton(selectWallAction);
		JToggleButton delete = Utils.createToggleButton(selectDeleteAction);

		toolBar.add(hamster);
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(turnHamsterAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(hamsterCornAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(corn);
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(wall);
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(delete);

		ButtonGroup group = new ButtonGroup();
		group.add(corn);
		group.add(hamster);
		group.add(wall);
		group.add(delete);

		toolBar.add(Box.createRigidArea(new Dimension(11, 11)));

		toolBar.add(Utils.createButton(zoomInAction));
		toolBar.add(Box.createRigidArea(new Dimension(2, 2)));
		toolBar.add(Utils.createButton(zoomOutAction));

		toolBar.add(Box.createRigidArea(new Dimension(11, 11)));
		toolBar.add(Utils.createButton(resetAction));
	}

	public void setSimulationPanel(SimulationPanel simulationPanel) {
		this.simulationPanel = simulationPanel;
		simulationPanel.addMouseListener(this);
		simulationPanel.addMouseMotionListener(this);
	}

	public void paint(Graphics2D g) {
		if (activeTool == null || !isDragging)
			return;
		if (activeTool.isDragTool()) {
			int minCol = Math.min(startCol, endCol);
			int maxCol = Math.max(startCol, endCol);
			int minRow = Math.min(startRow, endRow);
			int maxRow = Math.max(startRow, endRow);
			for (int i = minCol; i <= maxCol; i++) {
				if (i < 0 || i >= simulationModel.getTerrain().getWidth())
					continue;
				for (int j = minRow; j <= maxRow; j++) {
					if (j < 0 || j >= simulationModel.getTerrain().getHeight())
						continue;
					simulationPanel.drawImage(g, activeTool.getImage(), i, j);
				}
			}
		}
	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent e) {
		startCol = simulationPanel.getCol(e.getX());
		startRow = simulationPanel.getRow(e.getY());
		endCol = startCol;
		endRow = startRow;
		isDragging = true;
		simulationPanel.repaint();
	}

	public void mouseReleased(MouseEvent e) {
		if (activeTool != null)
			activeTool.done();
		isDragging = false;
		simulationPanel.repaint();
	}

	public void mouseDragged(MouseEvent e) {
		endCol = simulationPanel.getCol(e.getX());
		endRow = simulationPanel.getRow(e.getY());
		simulationPanel.repaint();
	}

	public void mouseMoved(MouseEvent e) {
		// TODO Auto-generated method stub

	}
}